///////////////////////////////////////////////////////////////////////////////
//                           Function defenitions                            //
///////////////////////////////////////////////////////////////////////////////

// These actions control the tutorials in which the player has no interaction with.
Trigger.prototype.WelcomeMessage = function(data)
{
	PushGUINotificationInTopCenter(1, "Welcome to the basic tutorial of the 0 A.D.!");
	this.DoAfterDelay(3000, "IntroductionMessage", {});
}

Trigger.prototype.IntroductionMessage = function(data)
{
	PushGUINotificationInTopCenter(1, "In this tutorial you will learn the basics of the gameplay of 0 A.D.");
	this.DoAfterDelay(6000, "CameraMovementTutorial1", {});
}

Trigger.prototype.CameraMovementTutorial1 = function(data)
{
	PushGUINotificationInTopCenter(1, "First, we try the camera movements. To move your camera, move the mouse pointer to the edge of the screen.");
	this.DoAfterDelay(8000, "CameraMovementTutorial2", {});
}

Trigger.prototype.CameraMovementTutorial2 = function(data)
{
	PushGUINotificationInTopCenter(1, "You can also move the camera using W,A,S,D or arrow keys on the keyboard, or by holding the middle mouse button and moving it.");
	this.DoAfterDelay(15000, "CameraMovementTutorial3", {});
}

Trigger.prototype.CameraMovementTutorial3 = function(data)
{
	PushGUINotificationInTopCenter(1, "To rotate the camera, hold Ctrl and one of the arrow keys.");
	this.DoAfterDelay(7000, "CameraMovementTutorial4", {});
}

Trigger.prototype.CameraMovementTutorial4 = function(data)
{
	PushGUINotificationInTopCenter(1, "To zoom, use the mouse scroll or + and - keys on the keyboard.");
	this.DoAfterDelay(7000, "IntroductionToUnits", {});
}

Trigger.prototype.IntroductionToUnits = function(data)
{
	PushGUINotificationInTopCenter(1, "Now it is time to work with units. Move the mouse pointer on one of the units present in the screen, and left-click to select it. Then, right-click on an open part of the terrain.");
	this.EnableIntervalTrigger("MovingUnitsTutorial");
	this.EnableTrigger("OnUnitIssuedOrder", "HandleOrders");
}

Trigger.prototype.IntroductionToResources = function(data)
{
	PushGUINotificationInTopCenter(1, "Resource gathering play a central role in 0 A.D.. There are four resources found in the game. Food, Wood, Stone and metal. You can find your current stockpile in the upper left section of the screen.");
	this.DoAfterDelay(15000, "MoreDescriptionAboutResources", {});
}

Trigger.prototype.MoreDescriptionAboutResources = function(data)
{
	PushGUINotificationInTopCenter(1, "Resources are required to train units, build structures and research technologies.");
	this.EnableIntervalTrigger("TutorialOnGatheringFood");
}

// These actions remind the players of what they should do.
Trigger.prototype.MovingUnitsTutorial = function(data)
{
	PushGUINotificationInTopCenter(1, "Select a unit by left clicking on it, and then, right-click on an open part of the terrain.");
}

Trigger.prototype.MovingAGroupOfUnitsTutorial = function(data)
{
	PushGUINotificationInTopCenter(1, "Hold the left button of the mouse and drag it to select a group of units. Then, order them to move.");
}

Trigger.prototype.TutorialOnGatheringFood = function(data)
{
	PushGUINotificationInTopCenter(1, "Food is gathered from farms, fruit trees and bushes, animals, fishes and farms. Select some of your units and right-click on the berry bushes in the east to gather food.");
}

Trigger.prototype.TutorialOnGatheringWood = function(data)
{
	PushGUINotificationInTopCenter(1, "Wood is gathered from trees. Select some of your units and right-click on a tree to gather wood.");
}

Trigger.prototype.TutorialOnGatheringStone = function(data)
{
	PushGUINotificationInTopCenter(1, "Stone is gathered from stone quarries. Select some of your units and right-click on the quarry in the south of your town to gather stone.");
}

Trigger.prototype.TutorialOnGatheringMetal = function(data)
{
	PushGUINotificationInTopCenter(1, "Metal is gathered from metal ores. Select some of your units and right-click on the rock in the north-east of your town to gather metal.");
}

Trigger.prototype.TutorialOnExploration = function(data)
{
	PushGUINotificationInTopCenter(1, "Notice that you can only see the immidiate surroundings of you town. To see further, select your rider and order him to move along the south-eastern road into the black area.");
}

Trigger.prototype.TutorialOnGatheringTreasure = function(data)
{
	PushGUINotificationInTopCenter(1, "Order your rider to gather the treasures. To do so, select your rider and right-click on one of the treasures near the road.");
}

// Thse actions set the next stage of the tutorial after the player has successfully done the previous ones

Trigger.prototype.HandleOrders = function(data)
{
	if (data.order == "Order.Walk")
		this.PlayerMovedAUnit(data);
	else if (data.order == "Order.Gather")
	{
		var type = GetEntityResourceType(data.target);
		if (type === undefined)
			return;
		else if (type.specific == "fruit")
			this.UnitOrderedToGatherFruit(data);
		else if (type.specific == "tree")
			this.UnitOrderedToGatherTree(data);
		else if (type.specific == "rock")
			this.UnitOrderedToGatherStoneQuarry(data);
		else if (type.specific == "ore")
			this.UnitOrderedToGatherMetalMine(data);
		else if (type.generic == "treasure")
			this.UnitOrderedToGatherTreasure(data);
	}
}

Trigger.prototype.HandleMultipleEntityOrders = function(data)
{
	if (data.order == "Order.Walk")
	{
		if (this.movingTutorial1Completed && !this.movingTutorial2Completed)
		{
			this.movingTutorial2Completed = true;
			this.DisableIntervalTrigger("MovingAGroupOfUnitsTutorial");
			PushGUINotificationInTopCenter(1, "Very Good.");
			this.DoAfterDelay(6000, "IntroductionToResources", {});
		}
	}
}

// This one is called when the player moves a unit.
Trigger.prototype.PlayerMovedAUnit = function(data)
{
	if (!this.movingTutorial1Completed)
	{
		this.movingTutorial1Completed = true;
		this.DisableIntervalTrigger("MovingUnitsTutorial");
		PushGUINotificationInTopCenter(1, "Good. You can also select a group of units by holding the left mouse button and dragging it. Select a group of units and order them to move.");
		this.EnableIntervalTrigger("MovingAGroupOfUnitsTutorial");
		this.EnableTrigger("OnUnitsIssuedOrder", "HandleMultipleEntityOrders");
	}
}

// This one is called when the player orders a unit to gather from fruit bushes
Trigger.prototype.UnitOrderedToGatherFruit = function(data)
{
	if (!this.gatheringFruitCompleted)
	{
		PushGUINotificationInTopCenter(1, "Excelent. Now your units will move and gather food from those bushes.");
		this.DisableIntervalTrigger("TutorialOnGatheringFood");
		
		this.EnableIntervalTrigger("TutorialOnGatheringWood");
		
		this.gatheringFruitCompleted = true;
	}
}

// This one is called when the player orders a unit to gather from trees
Trigger.prototype.UnitOrderedToGatherTree = function(data)
{
	if (!this.gatheringTreeCompleted)
	{
		PushGUINotificationInTopCenter(1, "Perfect. Now your units will move and gather wood from the trees.");
		this.DisableIntervalTrigger("TutorialOnGatheringWood");
		
		this.EnableIntervalTrigger("TutorialOnGatheringStone");
		
		this.gatheringTreeCompleted = true;
	}
}

// This one is called when the player orders a unit to gather from stone quarries
Trigger.prototype.UnitOrderedToGatherStoneQuarry = function(data)
{
	if (!this.gatheringStoneCompleted)
	{
		PushGUINotificationInTopCenter(1, "Good. Now your units will move and gather stone from the quarry.");
		this.DisableIntervalTrigger("TutorialOnGatheringStone");

		this.EnableIntervalTrigger("TutorialOnGatheringMetal");
		
		this.gatheringStoneCompleted = true;
	}
}

// This one is called when the player orders a unit to gather from metal mines
Trigger.prototype.UnitOrderedToGatherMetalMine = function(data)
{
	if (!this.gatheringMetalCompleted)
	{
		PushGUINotificationInTopCenter(1, "Very Good. Now your units will move and gather metal from the ore.");
		this.DisableIntervalTrigger("TutorialOnGatheringMetal");
		
		// Spawn a cavalry unit for the player in his/her civic center.
		var playerEntities = GetEntitiesByPlayer(1); // Get all of the player's entities
		for (var entity of playerEntities)
		{
			if (EntityHasClass(entity, "CivilCentre"))
			{
				BuildingSpawnUnits(entity, "skirmish/units/default_cavalry", 1);
				break;
			}
		}
		
		this.EnableIntervalTrigger("TutorialOnExploration");
		this.EnableRangeTrigger("StartTreasureTutorial");
		
		this.gatheringMetalCompleted = true;
	}
}

// This one is called when a unit gets near the market in the south-east
Trigger.prototype.StartTreasureTutorial = function(data)
{
	PushGUINotificationInTopCenter(1, "You have found some treasures. Treasures give large ammount of their representing resources instantly. Order your rider to gather the treasures.");
	this.DisableIntervalTrigger("TutorialOnExploration");
	this.DisableRangeTrigger("StartTreasureTutorial");
	
	this.EnableIntervalTrigger("TutorialOnGatheringTreasure");
}

// This one is called when the player orders a unit to gather from treasures
Trigger.prototype.UnitOrderedToGatherTreasure = function(data)
{
	if (!this.gatheringTreasureCompleted)
	{
		PushGUINotificationInTopCenter(1, "Outstanding. This tutorial is now done.");
		this.DisableIntervalTrigger("TutorialOnGatheringTreasure");
		
		this. gatheringTreasureCompleted = true;
		
		this.DoAfterDelay(4000, "Victory", {});
	}
}

Trigger.prototype.Victory = function(data)
{
	SetPlayerWon(1);
}

///////////////////////////////////////////////////////////////////////////////
//                            Startup Actions                                //
///////////////////////////////////////////////////////////////////////////////

// This function is run in the start of the game. We should initialize everything here.
Trigger.prototype.InitGame = function(data)
{
	// Our trigger variables should be defined here or they will be lost
	// after a save.
	this.movingTutorial1Completed = false;
	this.movingTutorial2Completed = false;
	this.gatheringFruitCompleted = false;
	this.gatheringTreeCompleted = false;
	this.gatheringStoneCompleted = false;
	this.gatheringMetalCompleted = false;
	this.gatheringTreasureCompleted = false;
	
	// Start the first tutorial after 2 seconds.
	this.DoAfterDelay(2000, "WelcomeMessage", {});

	///////////////////////////////////////////////////////////////////////////
	//                           Register Triggers                           //
	///////////////////////////////////////////////////////////////////////////

	// For a trigger to work, actions should be bound to an event.
	// Interval triggers, when active, are run infinitely each interval.
	this.RegisterOnIntervalTrigger("MovingUnitsTutorial", 10000, 12000, false); 		  // The first paramete is the function
																							  // call in each interval.
	this.RegisterOnIntervalTrigger("MovingAGroupOfUnitsTutorial", 10000, 10000, false); // The first number is the number of
																							  // miliseconds before the start of the
																							  // interval. The second is the number
																							  // of miliseconds between each interval
	this.RegisterOnIntervalTrigger("TutorialOnGatheringFood", 5000, 10000, false);      // "false" means that it is disabled by default
	this.RegisterOnIntervalTrigger("TutorialOnGatheringWood", 5000, 10000, false);
	this.RegisterOnIntervalTrigger("TutorialOnGatheringStone", 5000, 10000, false);
	this.RegisterOnIntervalTrigger("TutorialOnGatheringMetal", 5000, 10000, false);
	this.RegisterOnIntervalTrigger("TutorialOnExploration", 5000, 10000, false);

	// the query data to be used for the following range query
	var queryData = {
		"maxRange": 40, // maximum range
		"players": [1], // only accept results from player 1
		"requiredComponent": IID_UnitAI, // require the results to implement the UnitAI
		"action": "StartTreasureTutorial", // function to execute
		"enabled": true, // enabled
	}
	// Range triggers are run when a unit enters or leaves the circle around a entity 226 (the market)
	this.RegisterRangeTrigger(226, queryData, "StartTreasureTutorial");
	
	this.RegisterOnIntervalTrigger("TutorialOnGatheringTreasure", 5000, 10000, false);
	
	// Normal triggers. The first parameter is the "event". The second one is the action, and the third one is the default enability
	this.RegisterTrigger("OnUnitIssuedOrder", "HandleOrders", false);
	this.RegisterTrigger("OnUnitsIssuedOrder", "HandleMultipleEntityOrders", false);
}

